/* deleteop.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001 Ralf Hoffmann.
 * You can contact me at: ralf.hoffmann@epost.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "deleteop.h"
#include "listermode.h"
#include "normalmode.h"
#include "worker.h"

const char *DeleteOp::name="DeleteOp";

DeleteOp::DeleteOp(Worker *tworker):FunctionProto(tworker)
{
}

DeleteOp::~DeleteOp()
{
}

DeleteOp*
DeleteOp::duplicate()
{
  DeleteOp *ta=new DeleteOp(worker);
  return ta;
}

bool
DeleteOp::isName(const char *str)
{
  if(strcmp(str,name)==0) return true; else return false;
}

const char *
DeleteOp::getName()
{
  return name;
}

int
DeleteOp::run(ActionMessage *msg)
{
  ListerMode *lm1;
  am=msg;
  if(msg->mode!=msg->AM_MODE_DNDACTION) {
    Lister *l1=worker->getActiveLister();
    if(l1!=NULL) {
      startlister=l1;
      lm1=startlister->getActiveMode();
      if(lm1!=NULL)
        if(lm1->isType("NormalMode")==true)
          normalmodedelete();
    }
  }
  return 0;
}

const char *
DeleteOp::getDescription()
{
  return catalog.getLocaleCom(25);
}

int
DeleteOp::normalmodedelete()
{
  struct NM_deleteorder delorder;
  NormalMode *nm1=NULL;
  ListerMode *lm1=NULL;
  struct NM_specialsource *specialsource=NULL;
  
  if(startlister==NULL) return 1;
  lm1=startlister->getActiveMode();
  if(lm1==NULL) return 1;
  if(lm1->isType("NormalMode")!=true) return 1;
  nm1=(NormalMode*)lm1;
  
  if(am->mode==am->AM_MODE_NORMAL) {
    delorder.source=delorder.NM_ALLENTRIES;

    delorder.delmode=delorder.NM_DELMODE_NORMAL;
    delorder.dirdel=delorder.NM_DIRDELETE_NORMAL;
    delorder.dowin=new DeleteOpWin(worker->getAGUIX());
    nm1->deletef(&delorder);
    delete delorder.dowin;
    if(delorder.source==delorder.NM_SPECIAL) {
      if(specialsource!=NULL) _freesafe(specialsource);
      delete delorder.sources;
    }
  }
  return 0;
}

/*********************
 * Class DeleteOpWin *
 *********************/

DeleteOpWin::DeleteOpWin(AGUIX*taguix)
{
  this->aguix=taguix;
  source=dupstring("");
  deleted_files=0;
  files=0;
  deleted_dirs=0;
  dirs=0;
  
  window=NULL;
}

DeleteOpWin::~DeleteOpWin()
{
  if(source!=NULL) _freesafe(source);
  if(window!=NULL) close();
}

int
DeleteOpWin::open()
{
  int w,h;
  int tx,ty;
  Text *ttext;
  BevelBox *bb;

  close();
  window=new AWindow(aguix);
  w=400;
  h=10;
  window->create(NULL,10,10,w,h,0,catalog.getLocaleCom(25));
  tx=5;
  ty=5;
  sourcetext=(Text*)window->add(new Text(aguix,tx,ty,"",1));
  ty+=sourcetext->getHeight()+5;
  bb=(BevelBox*)window->add(new BevelBox(aguix,tx,ty,w-2*tx,2,0));
  ty+=bb->getHeight()+5;

  ttext=(Text*)window->add(new Text(aguix,tx,ty,catalog.getLocale(142),1));
  tx+=ttext->getWidth()+5;
  files2gotext=(Text*)window->add(new Text(aguix,tx,ty,"",1));
  tx=5;
  ty+=ttext->getHeight()+5;
  ttext=(Text*)window->add(new Text(aguix,tx,ty,catalog.getLocale(143),1));
  tx+=ttext->getWidth()+5;
  dirs2gotext=(Text*)window->add(new Text(aguix,tx,ty,"",1));
  tx=5;
  ty+=ttext->getHeight()+5;

  gsb=(SolidButton*)window->add(new SolidButton(aguix,tx+1,ty+1,w-2*(tx+1),"",2,3,0));
  bb=(BevelBox*)window->add(new BevelBox(aguix,tx,ty,w-2*tx,gsb->getHeight()+2,1));
  gsb->toFront();
  sbw=gsb->getWidth();
  ty+=bb->getHeight()+5;

  bb=(BevelBox*)window->add(new BevelBox(aguix,tx,ty,w-2*tx,2,0));
  ty+=bb->getHeight()+5;
  cb=(Button*)window->add(new Button(aguix,tx,ty,
                                     (strlen(catalog.getLocale(8))+2)*aguix->getCharWidth(),
                                     catalog.getLocale(8),1,0,0));
  cb->move(w/2-cb->getWidth()/2,cb->getY());
  ty+=cb->getHeight()+5;
  h=ty;
  window->resize(w,h);
  window->setMaxSize(w,h);
  window->setMaxSize(w,h);
  window->show();
  return 0;
}

void
DeleteOpWin::close()
{
  if(window!=NULL) {
    window->close();
    delete window;
  }
  window=NULL;
}

void
DeleteOpWin::set_files_to_delete(long nfiles)
{
  if(nfiles>0) files=nfiles;
  else files=0;
  update_files2gotext=true;
}

void
DeleteOpWin::set_dirs_to_delete(long ndirs)
{
  if(ndirs>0) dirs=ndirs;
  else dirs=0;
  update_dirs2gotext=true;
}

void
DeleteOpWin::dir_finished()
{
  deleted_dirs++;
  update_dirs2gotext=true;
}

void
DeleteOpWin::file_finished()
{
  // inc counter
  deleted_files++;
  update_files2gotext=true;
}

void
DeleteOpWin::setfilename(char *name)
{
  if(source!=NULL) _freesafe(source);
  if(name!=NULL)
    source=dupstring(name);
  else
    source=dupstring("");
  update_sourcetext=true;
}

int
DeleteOpWin::redraw()
{ /* this functions will redraw the window AND process events for this window
   * returns values:
   *  0 for regular exit
   *  1 when clicked Cancel-Button or closed window
   *  other not implemented */
  int returnvalue=0;
  char tstr[1024],*tstr2;
  float rate;
  bool updategsb=false;

  aguix->Flush();
  if(update_sourcetext==true) {
    tstr2=(char*)_allocsafe(strlen(catalog.getLocale(141))+strlen(source)+1);
    if(source!=NULL)
      sprintf(tstr2,catalog.getLocale(141),source);
    else
      sprintf(tstr2,catalog.getLocale(141),"");
    sourcetext->setText(tstr2);
    _freesafe(tstr2);
    /* no need for explicit redraw */
    update_sourcetext=false;
  }
  if(update_files2gotext==true) {
    sprintf(tstr,"%ld",files-deleted_files);
    files2gotext->setText(tstr);
    update_files2gotext=false;
    updategsb=true;
  }
  if(update_dirs2gotext==true) {
    sprintf(tstr,"%ld",dirs-deleted_dirs);
    dirs2gotext->setText(tstr);
    update_dirs2gotext=false;
    updategsb=true;
  }
  if(updategsb==true) {
    if((files+dirs)>0)
      rate=(float)(deleted_files+deleted_dirs)/(float)(files+dirs);
    else
      rate=0.0;
    sprintf(tstr,"%.4g %%",rate*100);
    gsb->setText(tstr);
    gsb->resize((int)(sbw*rate),gsb->getHeight());
  }
  /* update complete
     now check for X-messages */
  AGMessage *msg;
  do {
    msg=aguix->GetMessage(NULL);
    if(msg!=NULL) {
      switch(msg->type) {
        case AG_CLOSEWINDOW:
          if(msg->closewindow.window==window->getWindow()) returnvalue=1;
          break;
        case AG_BUTTONCLICKED:
          if(msg->button.button==cb) returnvalue=1;
          break;
      }
      aguix->ReplyMessage(msg);
    }
  } while(msg!=NULL);
  return returnvalue;
}

void
DeleteOpWin::setmessage(char *msg)
{
  if(msg!=NULL) sourcetext->setText(msg);
  else sourcetext->setText("");
}

void
DeleteOpWin::dec_file_counter(unsigned long f)
{
  files-=f;
  update_files2gotext=true;
}

void
DeleteOpWin::dec_dir_counter(unsigned long d)
{
  dirs-=d;
  update_dirs2gotext=true;
}

