/* partspace.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001 Ralf Hoffmann.
 * You can contact me at: ralf.hoffmann@epost.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "partspace.h"

#include "wdefines.h"

PartSpace::PartSpace()
{
  filename=NULL;
}

PartSpace::~PartSpace()
{
  if(filename!=NULL) _freesafe(filename);
}

PartSpace::PartSpace(char*name)
{
  filename=NULL;
  readSpace(name);
}

int PartSpace::readSpace(char*name)
{
#ifdef HAVE_SYS_STATVFS_H
  struct statvfs fsstatistic;
#else
  struct statfs fsstatistic;
#endif

  if(filename!=NULL) _freesafe(filename);
  filename=NULL;

#ifdef HAVE_STATVFS
  if(statvfs(name,&fsstatistic)==0) {
    blocksize=fsstatistic.f_frsize;
    space=fsstatistic.f_blocks;
    freespace=fsstatistic.f_bavail;
#else
  if(statfs(name,&fsstatistic)==0) {
    blocksize=fsstatistic.f_bsize;
    space=fsstatistic.f_blocks;
    freespace=fsstatistic.f_bavail;
#endif
    filename=dupstring(name);
  } else return 1;
  return 0;
}

long PartSpace::getBlocksize()
{
  return blocksize;
}

long PartSpace::getFreeSpace()
{
  return freespace;
}

long PartSpace::getSpace()
{
  return space;
}

// H stands for human readable, means in B/KB/MB/GB when great enough

long PartSpace::getFreeSpaceH(char **unit_return)
{
  double dfree;
  char buffer[3];

  dfree=(double)freespace*(double)blocksize;
  if(dfree>10.0*1024.0*1024.0*1024.0) {
    dfree+=512.0*1024.0*1024.0;
    dfree/=1024.0*1024.0*1024.0;
    strcpy(buffer,"GB");
  } else if(dfree>10485760) {
    dfree+=524288;
    dfree/=1048576;
    strcpy(buffer,"MB");
  } else if(dfree>10240) {
    dfree+=512;
    dfree/=1024;
    strcpy(buffer,"KB");
  } else {
    strcpy(buffer,"B");
  }
  if(unit_return!=NULL) {
    strcpy(*unit_return,buffer);
  }
  return (long)dfree;
}

long PartSpace::getSpaceH(char **unit_return)
{
  double dsize;
  char buffer[3];

  dsize=((double)space)*((double)blocksize);
  if(dsize>10.0*1024.0*1024.0*1024.0) {
    dsize+=512.0*1024.0*1024.0;
    dsize/=1024.0*1024.0*1024.0;
    strcpy(buffer,"GB");
  } else if(dsize>10485760) {
    dsize+=524288.0;
    dsize/=1048576.0;
    strcpy(buffer,"MB");
  } else if(dsize>10240) {
    dsize+=512;
    dsize/=1024;
    strcpy(buffer,"KB");
  } else {
    strcpy(buffer,"B");
  }
  if(unit_return!=NULL) {
    strcpy(*unit_return,buffer);
  }
  return (long)dsize;
}

